# -*- coding: utf-8 -*-
# @Time    : 2020/9/14 16:30
# @Author  :

from elasticsearch import Elasticsearch, helpers, ConnectionPool
from .hbase_helper import HBaseHelper
from .elasticsearch_helper import bulk_insearch_es

COLUMN = "info"


class HBaseEs(object):
    def __init__(self, hbase_host, es_host_list=[], user_tuple=()):
        self.hbase_host = hbase_host
        self.es_host_list = es_host_list
        self.user_tuple = user_tuple
        self.__conneteshbase()

    def __conneteshbase(self):
        self.es = Elasticsearch(self.es_host_list,
                                http_auth=self.user_tuple, timeout=180, max_retries=20,
                                retry_on_timeout=True, connection_pool_class=ConnectionPool)
        self.hbase = HBaseHelper(host=self.hbase_host)

    def __es_bulk(self, _op_type, index, data_list):
        """
        es批量操作方法
        :param _op_type: 操作类型
        :param index: 相应索引
        :param data_list: 相应数据
        :return:
        """
        bulk_list = [
            {
                '_op_type': _op_type,
                '_index': index,
                '_id': da.get("id"),
                "doc": da
            } for da in data_list

        ]
        return bulk_list

    def get(self, index, body, columns=[], prefix="", is_scroll=0, not_scroll_first=0, scroll="500m", sid="",
            size=100, only_es=0):
        """

        :param index:索引
        :param body: es dsl语法
        :param columns: 需要获取的字段如 ["c_url","kw"]
         :param prefix:前缀:dw,dm,ods
        :param is_scroll:是否支持滚动搜索 1支持 0不支持
        :param not_scroll_first: 不是第一次滚动 1不是第一次滚动 0是第一次滚动
        :param scroll: 滚动深度(默认200m)
        :param sid: 滚动游标
        :param size:滚动查询需要返回的条数
        :return:
        """
        scroll_size = 0
        body["_source"] = False
        if only_es == 1:
            body["_source"] = columns
        body["track_total_hits"] = True
        if is_scroll != 1:
            result = self.es.search(index=index, body=body)
        else:

            if not_scroll_first != 1:
                result = self.es.search(
                    index=index,
                    scroll=scroll,
                    size=size,
                    # es中的查询语法
                    body=body
                )
                sid = result['_scroll_id']
            else:
                result = self.es.scroll(scroll_id=sid, scroll=scroll)
            scroll_size = len(result['hits']['hits'])
        hits = result.get('hits')
        aggregations = result.get('aggregations', 0)
        total = hits.get("total")
        datas = hits.get('hits')
        sort = []
        if datas:
            sort = datas[-1].get("sort")
            rk_list = [da.get('_id') for da in datas]
        else:
            try:
                self.es.clear_scroll(
                    scroll_id=sid)
            except:
                pass
            sid = ""
            rk_list = []
        if only_es == 1:
            return {
                "data": {"total": total, "list": datas, "aggregations": aggregations, "sid": sid,
                         "scroll_size": scroll_size, "sort": sort}, 'msg': 'successful',
                'status': 200}
        if columns:
            columns = ["{column}:{k}".format(column=COLUMN, k=k) for k in columns]
        hbase_dict = self.hbase.query_multi_lines(row_keys=rk_list, index=index, prefix=prefix,
                                                  columns=columns)
        nw_list = []
        for k, v in hbase_dict.items():
            k_dict = {"id": k}
            data_dict = {k1.replace("%s:" % COLUMN, ""): v1 for k1, v1 in v.items()}
            data_dict.update(k_dict)
            nw_list.append(data_dict)
        return {
            "data": {"total": total, "list": nw_list, "aggregations": aggregations, "sid": sid,
                     "scroll_size": scroll_size, "sort": sort}, 'msg': 'successful',
            'status': 200}

    def add(self, index, prefix="", p_key_name="id", data_list=[], is_cover=1, only_es=0):
        """
         :param index:索引
         :param prefix:前缀:dw,dm,ods
        :param p_key_name:
        :param data_list:
        :param is_cover:是否覆盖默认为1表示覆盖 0表示不覆盖
        :return:
        """
        if only_es == 1:
            pass
        else:
            if is_cover == 0:
                rk_list = [da.get("id") for da in data_list]
                hbase_dict = self.hbase.query_multi_lines(row_keys=rk_list, index=index, prefix=prefix,
                                                          columns=[])
                data_list = [da for da in data_list if da.get("id") not in hbase_dict]
                data_hbase_list = [{k: str(v) for k, v in da.items()} for da in data_list]
            else:
                data_hbase_list = [{k: str(v) for k, v in da.items()} for da in data_list]

            self.hbase.batch_put(row_key_name=p_key_name, index=index, column=COLUMN, datas=data_hbase_list,
                                 prefix=prefix,
                                 batch_size=2000)
        bulk_insearch_es(self.es, p_key_name, datas=data_list, index=index)
        return {'msg': 'successful', 'status': 200}

    def update(self, index, prefix="", p_key_name="id", data_list=[], is_cover=0, only_es=0,
               exist_update_not_exist_index=0):
        """
        :param index:索引
         :param prefix:前缀:dw,dm,ods
        :param is_cover: 是否覆盖
        :param only_es: 是否只对es更新操作 only_es=1只对es进行更新操作
        :param exist_update_not_exist_index: 是否存在更新不存在插入 exist_update_not_exist_index=1 存在更新不存在插入
        :param p_key_name:
        :param data_list:
        :return:
        """
        _op_type = 'update'
        if is_cover == 1:
            _op_type = 'index'
            if only_es == 1:
                pass
            else:
                row_keys = [p.get(p_key_name) for p in data_list]
                self.hbase.batch_delete(row_keys, index=index)
        else:
            if exist_update_not_exist_index == 1:
                rk_list = [da.get("id") for da in data_list]
                # # 是否存在列表
                data_list_exit_list = []
                data_list_not_exit_list = []
                bulk_list = []
                result = self.es.mget(body={"ids": rk_list}, index=index)
                es_ids = [da.get("_id") for da in result.get("docs") if da.get('found')]
                for da in data_list:
                    id_ = da.get("id")
                    if id_ not in es_ids:
                        data_list_not_exit_list.append(da)
                    else:
                        data_list_exit_list.append(da)
                bulk_list.extend(self.__es_bulk('index', index, data_list_not_exit_list))
                bulk_list.extend(self.__es_bulk('update', index, data_list_exit_list))
                helpers.bulk(self.es, bulk_list)
                data_hbase_list = [{k: str(v) for k, v in da.items()} for da in data_list]
                self.hbase.batch_put(row_key_name=p_key_name, index=index, column=COLUMN, datas=data_hbase_list,
                                     prefix=prefix,
                                     batch_size=2000)
                return {
                    'msg': 'update successful', 'status': 200}
        if only_es == 1:
            pass
        else:
            data_hbase_list = [{k: str(v) for k, v in da.items()} for da in data_list]
            self.hbase.batch_put(row_key_name=p_key_name, index=index, column=COLUMN, datas=data_hbase_list,
                                 prefix=prefix,
                                 batch_size=2000)
        bulk_list = self.__es_bulk(_op_type, index, data_list)
        helpers.bulk(self.es, bulk_list)

        return {
            'msg': 'update successful', 'status': 200}

    def delete(self, index, prefix="", p_key_name="id", data_list=[], only_es=0):
        """
        :param index:索引
         :param prefix:前缀:dw,dm,ods
        :param p_key_name:
        :param data_list:
        :return:
        """
        _op_type = 'delete'
        if only_es == 1:
            pass
        else:
            row_keys = [p.get(p_key_name) for p in data_list]
            self.hbase.batch_delete(row_keys, index=index, prefix=prefix)
        bulk_list = [
            {
                '_op_type': _op_type,
                '_index': index,
                '_id': da.get("id"),
            } for da in data_list

        ]
        helpers.bulk(self.es, bulk_list)

        return {
            'msg': 'delete successful', 'status': 200}
